package dev.shadowsoffire.apotheosis.adventure;

import java.util.regex.Pattern;
import javax.annotation.Nullable;
import java.util.HashMap;
import java.util.ArrayList;
import net.minecraft.world.level.Level;
import net.minecraft.resources.ResourceKey;
import net.minecraft.world.level.WorldGenLevel;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import net.minecraft.ResourceLocationException;
import net.minecraft.world.item.Items;
import net.minecraftforge.registries.ForgeRegistries;
import dev.shadowsoffire.placebo.reload.DynamicHolder;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityRegistry;
import java.util.Locale;
import dev.shadowsoffire.placebo.config.Configuration;
import net.minecraft.world.item.Item;
import java.util.function.Supplier;
import net.minecraft.sounds.SoundEvent;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityClamp;
import dev.shadowsoffire.apotheosis.adventure.boss.BossEvents;
import org.apache.commons.lang3.tuple.Pair;
import dev.shadowsoffire.apotheosis.adventure.loot.LootCategory;
import java.util.Map;
import net.minecraft.resources.ResourceLocation;
import java.util.List;

public class AdventureConfig
{
    public static final List<ResourceLocation> DIM_WHITELIST;
    public static final Map<ResourceLocation, LootCategory> TYPE_OVERRIDES;
    public static final Map<ResourceLocation, Pair<Float, BossEvents.BossSpawnRules>> BOSS_SPAWN_RULES;
    public static final List<LootPatternMatcher> AFFIX_ITEM_LOOT_RULES;
    public static final List<LootPatternMatcher> GEM_LOOT_RULES;
    public static final Map<ResourceLocation, RarityClamp> GEM_DIM_RARITIES;
    public static final List<LootPatternMatcher> AFFIX_CONVERT_LOOT_RULES;
    public static final Map<ResourceLocation, RarityClamp> AFFIX_CONVERT_RARITIES;
    public static boolean curseBossItems;
    public static float bossAnnounceRange;
    public static float bossAnnounceVolume;
    public static boolean bossAnnounceIgnoreY;
    public static int bossSpawnCooldown;
    public static boolean bossAutoAggro;
    public static boolean bossGlowOnSpawn;
    public static List<SoundEvent> bossAnnounceSounds;
    public static float spawnerValueChance;
    public static float randomAffixItem;
    public static float gemDropChance;
    public static float gemBossBonus;
    public static boolean disableQuarkOnAffixItems;
    public static Supplier<Item> torchItem;
    public static boolean cleaveHitsPlayers;
    
    public static void load(final Configuration c) {
        c.setTitle("Apotheosis Adventure Module Config");
        AdventureConfig.TYPE_OVERRIDES.clear();
        AdventureConfig.TYPE_OVERRIDES.putAll(AdventureModule.IMC_TYPE_OVERRIDES);
        final String[] stringList;
        final String[] overrides = stringList = c.getStringList("Equipment Type Overrides", "affixes", new String[] { "minecraft:iron_sword|sword", "minecraft:shulker_shell|none" }, "A list of type overrides for the affix loot system.  Format is <itemname>|chance|<type>.\nValid types are: none, sword, trident, shield, heavy_weapon, pickaxe, shovel, crossbow, bow");
        for (int length = stringList.length, i = 0; i < length; ++i) {
            final String s = stringList[i];
            final String[] split = s.split("\\|");
            try {
                final LootCategory type = LootCategory.byId(split[1].toLowerCase(Locale.ROOT));
                if (type.isArmor()) {
                    throw new UnsupportedOperationException("Cannot override an item to an armor type.");
                }
                AdventureConfig.TYPE_OVERRIDES.put(new ResourceLocation(split[0]), type);
            }
            catch (final Exception e) {
                AdventureModule.LOGGER.error("Invalid type override entry: " + s + " will be ignored!");
                e.printStackTrace();
            }
        }
        AdventureConfig.randomAffixItem = c.getFloat("Random Affix Chance", "affixes", AdventureConfig.randomAffixItem, 0.0f, 1.0f, "The chance that a naturally spawned mob will be granted an affix item. 0 = 0%, 1 = 100%");
        AdventureConfig.gemDropChance = c.getFloat("Gem Drop Chance", "affixes", AdventureConfig.gemDropChance, 0.0f, 1.0f, "The chance that a mob will drop a gem. 0 = 0%, 1 = 100%");
        AdventureConfig.gemBossBonus = c.getFloat("Gem Boss Bonus", "affixes", AdventureConfig.gemBossBonus, 0.0f, 1.0f, "The flat bonus chance that bosses have to drop a gem, added to Gem Drop Chance. 0 = 0%, 1 = 100%");
        AdventureConfig.cleaveHitsPlayers = c.getBoolean("Cleave Players", "affixes", AdventureConfig.cleaveHitsPlayers, "If affixes that cleave can hit players (excluding the user).");
        String[] lootRules = c.getStringList("Affix Item Loot Rules", "affixes", new String[] { "minecraft:chests.*|0.35", ".*chests.*|0.3", "twilightforest:structures.*|0.3" }, """
                                                                                                                                                                                  Loot Rules, in the form of Loot Table Matchers, permitting affix items to spawn in loot tables.
                                                                                                                                                                                  The format for these is domain:pattern|chance and domain is optional.  Domain is a modid, pattern is a regex string, and chance is a float 0..1 chance for the item to spawn in any matched tables.
                                                                                                                                                                                  If you omit the domain, the format is pattern|chance, and the matcher will run for all domains.
                                                                                                                                                                                  The pattern MUST be a valid regex string, and should match the paths of desired loot tables under the specified domain.  Note: "Match Any Character" is ".*" (dot star) and not "*" (star).
                                                                                                                                                                                  If there is a match, an item has a chance to spawn in that loot table.""");
        AdventureConfig.AFFIX_ITEM_LOOT_RULES.clear();
        final String[] array = lootRules;
        for (int length2 = array.length, j = 0; j < length2; ++j) {
            final String s2 = array[j];
            try {
                AdventureConfig.AFFIX_ITEM_LOOT_RULES.add(LootPatternMatcher.parse(s2));
            }
            catch (final Exception e) {
                AdventureModule.LOGGER.error("Invalid affix item loot rule: " + s2 + " will be ignored");
                e.printStackTrace();
            }
        }
        lootRules = c.getStringList("Gem Loot Rules", "gems", new String[] { "minecraft:chests.*|0.25", ".*chests.*|0.20", "twilightforest:structures.*|0.20" }, "Loot Rules, in the form of Loot Table Matchers, permitting gems to spawn in loot tables.  See comment on \"Affix Item Loot Rules\" for description.");
        AdventureConfig.GEM_LOOT_RULES.clear();
        final String[] array2 = lootRules;
        for (int length3 = array2.length, k = 0; k < length3; ++k) {
            final String s2 = array2[k];
            try {
                AdventureConfig.GEM_LOOT_RULES.add(LootPatternMatcher.parse(s2));
            }
            catch (final Exception e) {
                AdventureModule.LOGGER.error("Invalid gem loot rule: " + s2 + " will be ignored");
                e.printStackTrace();
            }
        }
        lootRules = c.getStringList("Affix Convert Loot Rules", "affixes", new String[] { ".*blocks.*|0", ".*|0.35" }, "Loot Rules, in the form of Loot Table Matchers, permitting affixes to be added to any valid item. Here, the chance refers to the chance an item receives affixes. See comment on \"Affix Item Loot Rules\" for description.");
        AdventureConfig.AFFIX_CONVERT_LOOT_RULES.clear();
        final String[] array3 = lootRules;
        for (int length4 = array3.length, l = 0; l < length4; ++l) {
            final String s2 = array3[l];
            try {
                AdventureConfig.AFFIX_CONVERT_LOOT_RULES.add(LootPatternMatcher.parse(s2));
            }
            catch (final Exception e) {
                AdventureModule.LOGGER.error("Invalid affix convert loot rule: " + s2 + " will be ignored");
                e.printStackTrace();
            }
        }
        final String[] convertRarities = c.getStringList("Affix Convert Rarities", "affixes", new String[] { "overworld|common|rare", "the_nether|uncommon|epic", "the_end|rare|mythic", "twilightforest:twilight_forest|uncommon|epic" }, "Dimensional rarities for affix conversion (see \"Affix Convert Loot Rules\"), in the form of dimension|min|max. A dimension not listed uses all rarities.");
        AdventureConfig.AFFIX_CONVERT_RARITIES.clear();
        final String[] array4 = convertRarities;
        for (int length5 = array4.length, n = 0; n < length5; ++n) {
            final String s3 = array4[n];
            try {
                final String[] split2 = s3.split("\\|");
                final ResourceLocation dim = new ResourceLocation(split2[0]);
                final DynamicHolder<LootRarity> min = (DynamicHolder<LootRarity>)RarityRegistry.byLegacyId(split2[1]);
                final DynamicHolder<LootRarity> max = (DynamicHolder<LootRarity>)RarityRegistry.byLegacyId(split2[2]);
                AdventureConfig.AFFIX_CONVERT_RARITIES.put(dim, (RarityClamp)new RarityClamp.Simple((DynamicHolder)min, (DynamicHolder)max));
            }
            catch (final Exception e2) {
                AdventureModule.LOGGER.error("Invalid Affix Convert Rarity: " + s3 + " will be ignored");
                e2.printStackTrace();
            }
        }
        final String[] gemDimRarities = c.getStringList("Gem Dimensional Rarities", "gems", new String[] { "overworld|common|mythic", "the_nether|uncommon|mythic", "the_end|rare|mythic", "twilightforest:twilight_forest|uncommon|mythic" }, "Dimensional rarities for gem drops, in the form of dimension|min|max. A dimension not listed uses all rarities.");
        AdventureConfig.GEM_DIM_RARITIES.clear();
        final String[] array5 = gemDimRarities;
        for (int length6 = array5.length, n2 = 0; n2 < length6; ++n2) {
            final String s4 = array5[n2];
            try {
                final String[] split3 = s4.split("\\|");
                final ResourceLocation dim2 = new ResourceLocation(split3[0]);
                final DynamicHolder<LootRarity> min2 = (DynamicHolder<LootRarity>)RarityRegistry.byLegacyId(split3[1]);
                final DynamicHolder<LootRarity> max2 = (DynamicHolder<LootRarity>)RarityRegistry.byLegacyId(split3[2]);
                AdventureConfig.GEM_DIM_RARITIES.put(dim2, (RarityClamp)new RarityClamp.Simple((DynamicHolder)min2, (DynamicHolder)max2));
            }
            catch (final Exception e3) {
                AdventureModule.LOGGER.error("Invalid Gem Dimensional Rarity: " + s4 + " will be ignored");
                e3.printStackTrace();
            }
        }
        AdventureConfig.disableQuarkOnAffixItems = c.getBoolean("Disable Quark Tooltips for Affix Items", "affixes", true, "If Quark's Attribute Tooltip handling is disabled for affix items");
        final String torch = c.getString("Torch Placement Item", "affixes", "minecraft:torch", "The item that will be used when attempting to place torches with the torch placer affix.  Must be a valid item that places a block on right click.");
        AdventureConfig.torchItem = (() -> {
            try {
                final Item m = (Item)ForgeRegistries.ITEMS.getValue(new ResourceLocation(torch));
                return (m == Items.f_41852_) ? Items.f_42000_ : m;
            }
            catch (final Exception ex) {
                AdventureModule.LOGGER.error("Invalid torch item {}", (Object)torch);
                return Items.f_42000_;
            }
        });
        AdventureConfig.curseBossItems = c.getBoolean("Curse Boss Items", "bosses", AdventureConfig.curseBossItems, "If boss items are always cursed.  Enable this if you want bosses to be less overpowered by always giving them a negative effect.");
        AdventureConfig.bossAnnounceRange = c.getFloat("Boss Announce Range", "bosses", AdventureConfig.bossAnnounceRange, 0.0f, 1024.0f, "The range at which boss spawns will be announced.  If you are closer than this number of blocks (ignoring y-level), you will receive the announcement.");
        AdventureConfig.bossAnnounceVolume = c.getFloat("Boss Announce Volume", "bosses", AdventureConfig.bossAnnounceVolume, 0.0f, 1.0f, "The volume of the boss announcement sound. 0 to disable. This control is clientside.");
        AdventureConfig.bossAnnounceIgnoreY = c.getBoolean("Boss Announce Ignore Y", "bosses", AdventureConfig.bossAnnounceIgnoreY, "If the boss announcement range ignores y-level.");
        AdventureConfig.bossSpawnCooldown = c.getInt("Boss Spawn Cooldown", "bosses", AdventureConfig.bossSpawnCooldown, 0, 720000, "The time, in ticks, that must pass between any two natural boss spawns in a single dimension.");
        AdventureConfig.bossAutoAggro = c.getBoolean("Boss Auto-Aggro", "bosses", AdventureConfig.bossAutoAggro, "If true, invading bosses will automatically target the closest player.");
        AdventureConfig.bossGlowOnSpawn = c.getBoolean("Boss Glowing On Spawn", "bosses", AdventureConfig.bossGlowOnSpawn, "If true, bosses will glow when they spawn.");
        final String[] bossAnnounce = c.getStringList("Boss Announce Sounds", "bosses", new String[] { "block.end_portal.spawn" }, "List of sound effects to play when boss spawns are announced. This control is clientside.");
        AdventureConfig.bossAnnounceSounds.clear();
        final String[] array6 = bossAnnounce;
        for (int length7 = array6.length, n3 = 0; n3 < length7; ++n3) {
            final String s5 = array6[n3];
            try {
                AdventureConfig.bossAnnounceSounds.add(SoundEvent.m_262824_(new ResourceLocation(s5.trim())));
            }
            catch (final Exception e4) {
                AdventureModule.LOGGER.error("Invalid boss announce sound: " + s5 + " will be ignored");
                e4.printStackTrace();
            }
        }
        String[] dims = c.getStringList("Boss Spawn Dimensions", "bosses", new String[] { "minecraft:overworld|0.018|NEEDS_SKY", "minecraft:the_nether|0.025|ANY", "minecraft:the_end|0.018|SURFACE_OUTER_END", "twilightforest:twilight_forest|0.05|NEEDS_SURFACE" }, """
                                                                                                                                                                                                                                                                       Dimensions where bosses can spawn naturally, spawn chance, and spawn rules.
                                                                                                                                                                                                                                                                       Format is dimname|chance|rule, chance is a float from 0..1.
                                                                                                                                                                                                                                                                       Valid rules are visible here https://github.com/Shadows-of-Fire/Apotheosis/blob/1.19/src/main/java/shadows/apotheosis/adventure/boss/BossEvents.java#L174C27-L174C27""");
        AdventureConfig.BOSS_SPAWN_RULES.clear();
        final String[] array7 = dims;
        for (int length8 = array7.length, n4 = 0; n4 < length8; ++n4) {
            final String s6 = array7[n4];
            try {
                final String[] split4 = s6.split("\\|");
                AdventureConfig.BOSS_SPAWN_RULES.put(new ResourceLocation(split4[0]), Pair.of(Float.parseFloat(split4[1]), BossEvents.BossSpawnRules.valueOf(split4[2].toUpperCase(Locale.ROOT))));
            }
            catch (final Exception e5) {
                AdventureModule.LOGGER.error("Invalid boss spawn rules: " + s6 + " will be ignored");
                e5.printStackTrace();
            }
        }
        dims = c.getStringList("Generation Dimension Whitelist", "worldgen", new String[] { "overworld" }, "The dimensions that the deadly module will generate in.");
        AdventureConfig.DIM_WHITELIST.clear();
        final String[] array8 = dims;
        for (int length9 = array8.length, n5 = 0; n5 < length9; ++n5) {
            final String s6 = array8[n5];
            try {
                AdventureConfig.DIM_WHITELIST.add(new ResourceLocation(s6.trim()));
            }
            catch (final ResourceLocationException e6) {
                AdventureModule.LOGGER.error("Invalid dim whitelist entry: " + s6 + " will be ignored");
            }
        }
        AdventureConfig.spawnerValueChance = c.getFloat("Spawner Value Chance", "spawners", AdventureConfig.spawnerValueChance, 0.0f, 1.0f, "The chance that a Rogue Spawner has a \"valuable\" chest instead of a standard one. 0 = 0%, 1 = 100%");
    }
    
    public static boolean canGenerateIn(final WorldGenLevel world) {
        final ResourceKey<Level> key = (ResourceKey<Level>)world.m_6018_().m_46472_();
        return AdventureConfig.DIM_WHITELIST.contains(key.m_135782_());
    }
    
    static {
        DIM_WHITELIST = new ArrayList<ResourceLocation>();
        TYPE_OVERRIDES = new HashMap<ResourceLocation, LootCategory>();
        BOSS_SPAWN_RULES = new HashMap<ResourceLocation, Pair<Float, BossEvents.BossSpawnRules>>();
        AFFIX_ITEM_LOOT_RULES = new ArrayList<LootPatternMatcher>();
        GEM_LOOT_RULES = new ArrayList<LootPatternMatcher>();
        GEM_DIM_RARITIES = new HashMap<ResourceLocation, RarityClamp>();
        AFFIX_CONVERT_LOOT_RULES = new ArrayList<LootPatternMatcher>();
        AFFIX_CONVERT_RARITIES = new HashMap<ResourceLocation, RarityClamp>();
        AdventureConfig.curseBossItems = false;
        AdventureConfig.bossAnnounceRange = 96.0f;
        AdventureConfig.bossAnnounceVolume = 0.75f;
        AdventureConfig.bossAnnounceIgnoreY = false;
        AdventureConfig.bossSpawnCooldown = 3600;
        AdventureConfig.bossAutoAggro = false;
        AdventureConfig.bossGlowOnSpawn = true;
        AdventureConfig.bossAnnounceSounds = new ArrayList<SoundEvent>();
        AdventureConfig.spawnerValueChance = 0.11f;
        AdventureConfig.randomAffixItem = 0.075f;
        AdventureConfig.gemDropChance = 0.045f;
        AdventureConfig.gemBossBonus = 0.33f;
        AdventureConfig.disableQuarkOnAffixItems = true;
        AdventureConfig.torchItem = (() -> Items.f_42000_);
        AdventureConfig.cleaveHitsPlayers = false;
    }
    
    record LootPatternMatcher(@Nullable String domain, Pattern pathRegex, float chance) {
        public boolean matches(final ResourceLocation id) {
            return (this.domain == null || this.domain.equals(id.m_135827_())) && this.pathRegex.matcher(id.m_135815_()).matches();
        }
        
        public static LootPatternMatcher parse(final String s) throws Exception {
            final int pipe = s.lastIndexOf(124);
            final int colon = s.indexOf(58);
            final float chance = Float.parseFloat(s.substring(pipe + 1));
            final String domain = (colon == -1) ? null : s.substring(0, colon);
            final Pattern pattern = Pattern.compile(s.substring(colon + 1, pipe));
            return new LootPatternMatcher(domain, pattern, chance);
        }
        
        @Nullable
        public String domain() {
            return this.domain;
        }
    }
}
